@@ -1,28 +1,33 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/connect.c 1.18 1997/08/23 15:56:57 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/connect.c 2.14 1998/11/12 11:24:46 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2e.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.4.
   Handle WWWOFFLE connections received by the demon.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1996,97 Andrew M. Bishop
+  This file Copyright 1996,97,98 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
   ***************************************/
 
 
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include <sys/types.h>
+#include <time.h>
+#include <signal.h>
 #include <unistd.h>
 
 #include "wwwoffle.h"
+#include "misc.h"
 #include "config.h"
 #include "errors.h"
 
+/*+ The time that the program went online. +*/
+extern time_t OnlineTime;
 
 /*+ The server sockets that we listen on +*/
 extern int http_fd,             /*+ for the HTTP connections. +*/
@@ -31,7 +36,7 @@ extern int http_fd,             /*+ for the HTTP connections. +*/
 /*+ The proxy server that we use. +*/
 extern char *proxy;
 
-/*+ The online / offline status. +*/
+/*+ The online / offline / autodial status. +*/
 extern int online;
 
 /*+ The current number active servers +*/
@@ -51,19 +56,19 @@ extern int fetching;
 /*++++++++++++++++++++++++++++++++++++++
   Parse a request that comes from wwwoffle.
 
-  FILE *client The file pointer that corresponds to the wwwoffle connection.
+  int client The file descriptor that corresponds to the wwwoffle connection.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void CommandConnect(FILE *client)
+void CommandConnect(int client)
 {
  char *line=NULL;
 
- if(!(line=fgets_realloc(line,client)))
+ if(!(line=read_line_or_timeout(client,line)))
    {PrintMessage(Warning,"Nothing to read from the wwwoffle control socket [%!s]."); return;}
 
  if(strncmp(line,"WWWOFFLE ",9))
    {
-    PrintMessage(Warning,"Not a WWWOFFLE command.");
+    PrintMessage(Warning,"WWWOFFLE Not a command."); /* Used in audit-usage.pl */
     return;
    }
 
@@ -86,68 +91,78 @@ void CommandConnect(FILE *client)
 
     if(strcmp(password,PassWord))
       {
-       char *msg="WWWOFFLE Incorrect Password\n";
-       write(fileno(client),msg,strlen(msg));
-
-       PrintMessage(Warning,"Incorrect Password.");
-
+       write_string(client,"WWWOFFLE Incorrect Password\n");
+       PrintMessage(Warning,"WWWOFFLE Incorrect Password."); /* Used in audit-usage.pl */
        return;
       }
 
-    if(!(line=fgets_realloc(line,client)))
+    if(!(line=read_line_or_timeout(client,line)))
       {PrintMessage(Warning,"Unexpected end of wwwoffle control command [%!s]."); return;}
+
+    if(strncmp(line,"WWWOFFLE ",9))
+      {
+       PrintMessage(Warning,"WWWOFFLE Not a command."); /* Used in audit-usage.pl */
+       return;
+      }
    }
 
  if(!strncmp(&line[9],"ONLINE",6))
    {
-    char *msg;
-
-    if(online)
-       msg="WWWOFFLE Already Online\n";
+    if(online==1)
+       write_string(client,"WWWOFFLE Already Online\n");
     else
-       msg="WWWOFFLE Now Online\n";
+      {
+       write_string(client,"WWWOFFLE Now Online\n");
+       PrintMessage(Important,"WWWOFFLE Online."); /* Used in audit-usage.pl */
 
-    write(fileno(client),msg,strlen(msg));
-
-    if(!online)
-       PrintMessage(Important,"WWWOFFLE Online.");
+       DeleteLastTimeSpoolFile(NULL);
+      }
 
+    OnlineTime=time(NULL);
     online=1;
    }
+ else if(!strncmp(&line[9],"AUTODIAL",8))
+   {
+    if(online==-1)
+       write_string(client,"WWWOFFLE Already in Autodial Mode\n");
+    else
+      {
+       write_string(client,"WWWOFFLE Now In Autodial Mode\n");
+       PrintMessage(Important,"WWWOFFLE In Autodial Mode."); /* Used in audit-usage.pl */
+      }
+
+    OnlineTime=time(NULL);
+    online=-1;
+   }
  else if(!strncmp(&line[9],"OFFLINE",7))
    {
-    char *msg;
-
     if(!online)
-       msg="WWWOFFLE Already Offline\n";
+       write_string(client,"WWWOFFLE Already Offline\n");
     else
-       msg="WWWOFFLE Now Offline\n";
-
-    write(fileno(client),msg,strlen(msg));
-
-    if(online)
-       PrintMessage(Important,"WWWOFFLE Offline.");
+      {
+       write_string(client,"WWWOFFLE Now Offline\n");
+       PrintMessage(Important,"WWWOFFLE Offline."); /* Used in audit-usage.pl */
+      }
 
     online=0;
    }
  else if(!strncmp(&line[9],"FETCH",5))
    {
-    char *msg;
-
     if(fetch_fd!=-1)
-       msg="WWWOFFLE Already fetching.\n";
-    else if(!online)
-       msg="WWWOFFLE Must be online to fetch.\n";
+       write_string(client,"WWWOFFLE Already fetching.\n");
+    else if(online!=1)
+       write_string(client,"WWWOFFLE Must be online to fetch.\n");
     else
-       msg="WWWOFFLE Now Fetching.\n";
-
-    write(fileno(client),msg,strlen(msg));
-
-    PrintMessage(Important,"WWWOFFLE Fetch.");
-
-    if(fetch_fd==-1 && online)
       {
-       fetch_fd=fileno(client);
+       write_string(client,"WWWOFFLE Now Fetching.\n");
+       PrintMessage(Important,"WWWOFFLE Fetch."); /* Used in audit-usage.pl */
+      }
+    
+    if(fetch_fd==-1 && online==1)
+      {
+       RequestMonitoredPages();
+
+       fetch_fd=client;
        fetching=1;
 
        while(n_fetch_servers<MaxFetchServers && n_servers<MaxServers)
@@ -156,63 +171,52 @@ void CommandConnect(FILE *client)
    }
  else if(!strncmp(&line[9],"CONFIG",6))
    {
-    char *msg="WWWOFFLE Reading Configuration File.\n";
-    write(fileno(client),msg,strlen(msg));
+    write_string(client,"WWWOFFLE Reading Configuration File.\n");
+    PrintMessage(Important,"WWWOFFLE Re-reading Configuration File."); /* Used in audit-usage.pl */
 
-    PrintMessage(Important,"WWWOFFLE Re-reading Configuration File.");
-
-    if(ReadConfigFile(fileno(client)))
+    if(ReadConfigFile(client))
       {
        PrintMessage(Warning,"Error in configuration file; keeping old values.");
-
-       msg="WWWOFFLE Error Reading Configuration File.\n";
+       write_string(client,"WWWOFFLE Error Reading Configuration File.\n");
       }
     else
-       msg="WWWOFFLE Read Configuration File.\n";
-
-    write(fileno(client),msg,strlen(msg));
+       write_string(client,"WWWOFFLE Read Configuration File.\n");
 
     PrintMessage(Important,"WWWOFFLE Finished Re-reading Configuration File.");
    }
  else if(!strncmp(&line[9],"PURGE",5))
    {
-    char *msg="WWWOFFLE Purge Starting.\n";
-    write(fileno(client),msg,strlen(msg));
+    write_string(client,"WWWOFFLE Purge Starting.\n");
+    PrintMessage(Important,"WWWOFFLE Purge."); /* Used in audit-usage.pl */
 
-    if(PurgeUseMTime)
-       msg="(Using modification time.)\n";
-    else
-       msg="(Using last access time.)\n";
-    write(fileno(client),msg,strlen(msg));
-
-    PrintMessage(Important,"WWWOFFLE Purge.");
-
-    PurgeCache(fileno(client));
-
-    msg="WWWOFFLE Purge Finished.\n";
-    write(fileno(client),msg,strlen(msg));
+    PurgeCache(client);
 
+    write_string(client,"WWWOFFLE Purge Finished.\n");
     PrintMessage(Important,"WWWOFFLE Purge finished.");
    }
+ else if(!strncmp(&line[9],"KILL",4))
+   {
+    write_string(client,"WWWOFFLE Kill Signalled.\n");
+    PrintMessage(Important,"WWWOFFLE Kill."); /* Used in audit-usage.pl */
+
+    kill(getpid(),SIGTERM);
+   }
  else
    {
-    char *msg=(char*)malloc(strlen(line)+32);
-
     while(line[strlen(line)-1]=='\r' || line[strlen(line)-1]=='\n')
        line[strlen(line)-1]=0;
 
-    PrintMessage(Warning,"Unknown WWWOFFLE control command '%s'.",line);
-
-    sprintf(msg,"WWWOFFLE Unknown Command '%s'.",line);
-    write(fileno(client),msg,strlen(msg));
-
-    free(msg);
+    write_formatted(client,"WWWOFFLE Unknown Command '%s'.",line);
+    PrintMessage(Warning,"WWWOFFLE Unknown control command '%s'.",line); /* Used in audit-usage.pl */
    }
+
+ if(line)
+    free(line);
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Fork a server.
+  Fork a wwwoffles server.
 
   int client The file descriptor that the data comes in on.
 
@@ -221,7 +225,7 @@ void CommandConnect(FILE *client)
 
 void ForkServer(int client,int browser)
 {
- int pid;
+ pid_t pid;
 
  if((pid=fork())==-1)
     PrintMessage(Warning,"Cannot fork a server [%!s].");
@@ -231,7 +235,7 @@ void ForkServer(int client,int browser)
       {
        n_servers++;
 
-       if(online && !browser)
+       if(online==1 && !browser)
          {
           int i;
 
@@ -242,7 +246,8 @@ void ForkServer(int client,int browser)
           n_fetch_servers++;
          }
 
-       PrintMessage(Inform,"Forked wwwoffles -%s (pid=%d).",online?browser?"real":"fetch":"spool",pid);
+       /* Used in audit-usage.pl */
+       PrintMessage(Inform,"Forked wwwoffles -%s (pid=%d).",online==1?browser?"real":"fetch":online==-1?"autodial":"spool",pid);
       }
     else                         /* The child */
       {
