@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/errors.h 1.8 1997/08/23 15:56:58 amb Exp $
+  $Header: /home/amb/wwwoffle/v2.4/RCS/errors.h 2.4 1998/05/31 15:43:04 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2e.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.2.
   Error logging header file.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1996,97 Andrew M. Bishop
+  This file Copyright 1996,97,98 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -18,8 +18,9 @@
 
 typedef enum _ErrorLevel
 {
- Debug,                         /*+ For debugging, only if DEBUG is defined (not in syslog). +*/
- Inform,                        /*+ General information (not in syslog). +*/
+ ExtraDebug,                    /*+ For extra debugging (not in syslog). +*/
+ Debug,                         /*+ For debugging (debug in syslog). +*/
+ Inform,                        /*+ General information (info in syslog). +*/
  Important,                     /*+ Important information (notice in syslog). +*/
  Warning,                       /*+ A warning (warning in syslog). +*/
  Fatal                          /*+ A fatal error (err in syslog). +*/
@@ -28,7 +29,7 @@ ErrorLevel;
 
 /* In errors.c */
 
-void InitErrorHandler(char *name,int syslogable,int stderrable);
-void PrintMessage(ErrorLevel errlev,char* fmt, ...);
+void  InitErrorHandler(char *name,int syslogable,int stderrable);
+char *PrintMessage(ErrorLevel errlev,const char* fmt, ...);
 
 #endif /* ERRORS_H */
